%% Estimation of g-percentiles
%
% Steps of the program:
%   * 1. Preliminaries
%   * 2. Loads data
%   * 3. Defines g-percentiles
%   * 4. Estimates the thresholds and the average incomes at g-percentiles
%   * 5. Computes average incomes above g-percentiles
%   * 6. Exports the output
%
%% 1. Preliminaries
close all; clear all;

currentfolder=pwd;
% Path to load data
loadpath=[currentfolder,'/rawdata'];

%% 2. Loads data
disp('Loading data...');

cd(loadpath);
[~,sheets] = xlsfinfo('rawdata_bysheet');

for ii = 1:numel(sheets)
  
sheetname = char(sheets(ii));
disp(sheetname)

cd(loadpath);
[rawdata,txt] = xlsread('rawdata_bysheet',sheetname);
rawdata = rawdata(2:end,:);

if size(rawdata,2)>=4
    if ~strcmp(txt(end,4),'yav')
        rawdata = rawdata(:,1:3);
	end
elseif size(rawdata,2)>4
	rawdata = rawdata(:,1:4);
end
  
codespath=[currentfolder,'/codes'];
cd(codespath);
try
    [raw.thr,raw.p,raw.b,y_av,dirac,neg] = inputs_check(rawdata);
catch
    continue
end

if ~(length(dirac)==1)
      p0  = dirac(2);
	dirac = dirac(1);
else
    dirac = 0;
      p0  = NaN;
end

if ~(length(neg)==1)
    thr_neg = neg(1);
	 p_neg  = neg(2);
	av_neg  = neg(3);
else
    thr_neg = 0;
	 p_neg  = 0;
	av_neg  = 0;
end

clear rawdata;
cd(currentfolder);

%% 3. Defines g-percentiles
disp('Defining g-percentiles...');

gp1 = linspace(0,0.99,100);
gp2 = linspace(0.991,0.999,9);
gp3 = linspace(0.9991,0.9999,9);
gp4 = linspace(0.99991,0.99999,9);

gperc = [gp1,gp2,gp3,gp4]';

%% 4. Estimates the thresholds and the average incomes at g-percentiles
disp('Estimating the thresholds and the average incomes at g-percentiles...')

cd(codespath);
try
    [thr0,B_pos] = quantile(shift_p(gperc(gperc>=p_neg),p_neg),shift_thr(raw.thr,dirac),shift_p(raw.p,p_neg),shift_b(raw.thr,raw.b,dirac),shift_av(y_av,dirac,p_neg,av_neg),shift_p(p_neg+p0,p_neg));
catch
    continue
end
thr_pos = dirac+thr0;

B_pos = shift_b(thr_pos,B_pos,-dirac);
gperc_pos = gperc(gperc>=p_neg);

if p_neg>0
    try
        thr_neg = neg_part(gperc(gperc<p_neg)/p_neg,thr_neg,av_neg);
    catch
        continue
    end
else
    thr_neg = [];
end

thr = sort([thr_neg; thr_pos]);

%% 5. Computes average incomes above g-percentiles
disp('Computing average incomes above g-percentiles...')

av = zeros(size(gperc));

if isnan(p0)
    p0 = 0;
end

av(gperc>p_neg+p0)= thr(gperc>p_neg+p0).*B_pos(gperc_pos>p0);

rk = find(gperc>p_neg+p0,1);
for i=1:(rk-1)
    av(i) = (integral(@(p)(interp1(gperc,thr,p)),gperc(i),gperc(rk))+(1-gperc(rk))*av(rk))./(1-gperc(i));
end

B = NaN(size(gperc));
B(gperc>=p_neg) = B_pos;

%% 6. Exports the output
disp('Exporting the output...')

gperc = 100*gperc;
table = [gperc,thr,av,B];
outputpath=[currentfolder,'/output/bysheet'];
cd(outputpath);
filename = strcat('gperc_',sheetname,'.csv');

header = {'p','thr','ytop','b'};

fileID = fopen(filename, 'w');
if fileID == -1; error('Cannot open file: %s', filename); end
fprintf(fileID, '%s,', header{1:end-1});
fprintf(fileID, '%s\n', header{end});
fprintf(fileID, '%.3f,%.4f,%.4f,%.4f\n', table');
fclose(fileID);

end

disp('Routine finished.')