
%% Estimation of the generalized Pareto curve b(p) using income tax data
%
% Method: PCHIP interpolation
%
% Inputs:
%   * pp: table of percentiles (in [0,1]) corresponding to the thresholds in the tax
%   tabulation
%   * bb: table of (inverted) Pareto coefficients corresponding to the
%   thresholds in the tax tabulation
%   * P: table of points p (in [0,1]) where b(p) has to be calculated
%
% Output:
%   * B: table of approximative values taken by b at points of the input
%   table P, that is, B=b(P)
%
% Steps of the program:
%   1. Preliminaries
%   2. Interpolation of the Pareto curve above the lower threshold using
%   PCHIP method
%
%% 1. Preliminaries
% Defines function inputs and outputs.

function [BB] = pc6(thr,pp,bb,PP)

% Define beta
beta = log(((1-pp(1:(end-1))).*thr(1:(end-1)).*bb(1:(end-1)))./((1-pp(2:end)).*thr(2:end).*bb(2:end)));

BB = ones(size(PP));

pf = 1-(1-pp(end))/2;
bf = interp1([pp(end-1); pp(end)],[bb(end-1); bb(end)],pf,'linear','extrap');
bf = max(bf,1);
pp = [pp(:); pf];
bb = [bb(:); bf];

T = length(pp);
eps = 1e-16;
lc = 1/5;

%% 2. Approximation of the upper part

% 2.1 First bracket

%if 
%add condition

% Check if beta is consistent with convexity


%beta_func = @(p)

%if (integral(@(p)(1./((1-p).*env1(p,pp(1),pp(2),pp(3),pp(4),bb(1),bb(2),bb(3),bb(4)))),pp(2),pp(3))<=beta(2))&&(integral(@(p)(1./((1-p).*env2(p,pp(1),pp(2),pp(3),pp(4),bb(1),bb(2),bb(3),bb(4)))),pp(2),pp(3))>=beta(2))

%
Lp = [];
Lb = [];

for i=2:T-2
    i
    
    p1 = pp(i-1);
    p2 = pp(i);
    p3 = pp(i+1);
    p4 = pp(i+2);
    
    b1 = bb(i-1);
    b2 = bb(i);
    b3 = bb(i+1);
    b4 = bb(i+2);
    
    [BB,q1,q2,bq1,bq2] = f(p1,p2,p3,p4,b1,b2,b3,b4,Lp,Lb,beta(i),PP,BB);

    Lp = [Lp(:); p2; q1; q2];
    Lb = [Lb(:); b2; bq1; bq2];
    
end

    Lp = [Lp(:); pp(end-1)];
    Lb = [Lb(:); bb(end-1)];

function [BB,q1,q2,bq1,bq2] = f(p1,p2,p3,p4,b1,b2,b3,b4,Lp,Lb,beta0,PP,BB)
%function [BB,q1,q2] = f(p1,p2,p3,p4,b1,b2,b3,b4,Lp,Lb,beta0,PP,BB)

%bq1=0;
%bq2=0;
    
%delta = 0.01;
delta = 1/2;

env1 = @(p,p1,p2,p3,p4,b1,b2,b3,b4)(interp1([p1; p2; p3; p4],[b1; b2; b3; b4],p,'linear','extrap'));

intersection = @(p1,p2,p3,p4,b1,b2,b3,b4)(fzero(@(p)polyval(polyfit([p1 p2],[b1 b2],1)-polyfit([p3 p4],[b3 b4],1),p),0.5));

env2 = @(p,p1,p2,p3,p4,b1,b2,b3,b4)(interp1([p1; intersection(p1,p2,p3,p4,b1,b2,b3,b4); p4],[b1; polyval(polyfit([p1 p2],[b1 b2],1),intersection(p1,p2,p3,p4,b1,b2,b3,b4)); b4],p,'linear','extrap'));
    
    f_pc2 = @(p,p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb)(interp1([Lp(:); p2;q1;q2;p3],[Lb(:); b2;bq1;bq2;b3],p,'pchip','extrap'));
    
    f_beta2 = @(p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb)(quadgk(@(q)(1./((1-q).*f_pc2(q,p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb))),p2,p3,'MaxIntervalCount',10000));

    f_pc1 = @(p,p2,qc,p3,b2,bqc,b3,Lp,Lb)(interp1([Lp(:); p2;qc;p3],[Lb(:); b2;bqc;b3],p,'pchip','extrap'));
    
    f_beta1 = @(p2,qc,p3,b2,bqc,b3,Lp,Lb)(quadgk(@(q)(1./((1-q).*f_pc1(q,p2,qc,p3,b2,bqc,b3,Lp,Lb))),p2,p3,'MaxIntervalCount',10000));
    
    
q1 = (2/3)*p2+(1/3)*p3;
q2 = (1/3)*p2+(2/3)*p3;

BQ1 = @(bq2,b2,b3,param)(param*(b3+2*(bq2-b3))+(1-param)*(bq2+b2)/2);


if (b2-b1)/(p2-p1) < (b4-b3)/(p4-p3)
bq2_min = env2(q2,p1,p2,p3,p4,b1,b2,b3,b4);
    
if (integral(@(p)(1./((1-p).*env1(p,p1,p2,p3,p4,b1,b2,b3,b4))),p2,p3)<=beta0)&&(integral(@(p)(1./((1-p).*env2(p,p1,p2,p3,p4,b1,b2,b3,b4))),p2,p3)>=beta0)

    

if integral(@(p)(1./((1-p).*interp1([p2; q1; q2; p3],[b2; BQ1(bq2_min,b2,b3,1/2); bq2_min; b3],p,'linear','extrap'))),p2,p3)>beta0
    
    bq2_h = interp1([p2; p3],[b2; b3],q2,'linear','extrap');
    bq2_l = env2(q2,p1,p2,p3,p4,b1,b2,b3,b4);
    
    
    temp = f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_h,b2,b3,1/2),bq2_h,b3,Lp,Lb);
    
    while abs(temp-beta0)>eps
       t_loc = 0; 
       
    while f_beta2(p2,q1,q2,p3,b2,BQ1((1-t_loc)*bq2_h+t_loc*bq2_l,b2,b3,1/2),(1-t_loc)*bq2_h+t_loc*bq2_l,b3,Lp,Lb)<beta0
        t_loc = t_loc+delta;
    end
    
    bq2_l0 = bq2_l;
    bq2_h0 = bq2_h;
    
    bq2_h = (1-(t_loc-delta))*bq2_h0+(t_loc-delta)*bq2_l0;
    bq2_l = (1-t_loc)*bq2_h0+t_loc*bq2_l0;
    
    if abs(f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_l,b2,b3,1/2),bq2_l,b3,Lp,Lb)-beta0)<abs(f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_h,b2,b3,1/2),bq2_h,b3,Lp,Lb)-beta0)
        bq2_temp = bq2_l;
    else
        bq2_temp = bq2_h;
    end
    temp = f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_temp,b2,b3,1/2),bq2_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc2(PP((PP>=p2)&(PP<=p3)),p2,q1,q2,p3,b2,BQ1(bq2_temp,b2,b3,1/2),bq2_temp,b3,Lp,Lb);
    
    Lp = [Lp(:); p2; q1; q2];
    Lb = [Lb(:); b2; BQ1(bq2_temp,b2,b3,1/2); bq2_temp];
    bq2 = bq2_temp;
    bq1 = BQ1(bq2_temp,b2,b3,1/2);
    
%    end
    
elseif integral(@(p)(1./((1-p).*interp1([p2; q1; q2; p3],[b2; BQ1(bq2_min,b2,b3,3/4); bq2_min; b3],p,'linear','extrap'))),p2,p3)>beta0

    bq2_h = interp1([p2; p3],[b2; b3],q2,'linear','extrap');
    bq2_l = env2(q2,p1,p2,p3,p4,b1,b2,b3,b4);
    
    
    temp = f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_h,b2,b3,3/4),bq2_h,b3,Lp,Lb);
    
    while abs(temp-beta0)>eps
       t_loc = 0; 
       
    while f_beta2(p2,q1,q2,p3,b2,BQ1((1-t_loc)*bq2_h+t_loc*bq2_l,b2,b3,3/4),(1-t_loc)*bq2_h+t_loc*bq2_l,b3,Lp,Lb)<beta0
        t_loc = t_loc+delta;
    end
    
    bq2_l0 = bq2_l;
    bq2_h0 = bq2_h;
    
    bq2_h = (1-(t_loc-delta))*bq2_h0+(t_loc-delta)*bq2_l0;
    bq2_l = (1-t_loc)*bq2_h0+t_loc*bq2_l0;
    
    if abs(f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_l,b2,b3,3/4),bq2_l,b3,Lp,Lb)-beta0)<abs(f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_h,b2,b3,3/4),bq2_h,b3,Lp,Lb)-beta0)
        bq2_temp = bq2_l;
    else
        bq2_temp = bq2_h;
    end
    temp = f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_temp,b2,b3,3/4),bq2_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc2(PP((PP>=p2)&(PP<=p3)),p2,q1,q2,p3,b2,BQ1(bq2_temp,b2,b3,3/4),bq2_temp,b3,Lp,Lb);
    
    Lp = [Lp(:); p2; q1; q2];
    Lb = [Lb(:); b2; BQ1(bq2_temp,b2,b3,3/4);bq2_temp];
    
    bq1 = BQ1(bq2_temp,b2,b3,3/4);
    bq2 = bq2_temp;
    
else
    qc = intersection(p1,p2,p3,p4,b1,b2,b3,b4);
    if (qc<(1-lc)*p2+lc*p3)||(qc>(1-lc)*p3+lc*p2)
        qc = (p2+p3)/2;
    end
    
    bqc_h = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
    bqc_l = env2(qc,p1,p2,p3,p4,b1,b2,b3,b4);
    
    
    temp = f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb);
    
    while abs(temp-beta0)>eps
       t_loc = 0; 
       
    while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqc_h+t_loc*bqc_l,b3,Lp,Lb)<beta0
        t_loc = t_loc+delta;
    end
    
    bqc_l0 = bqc_l;
    bqc_h0 = bqc_h;
    
    bqc_h = (1-(t_loc-delta))*bqc_h0+(t_loc-delta)*bqc_l0;
    bqc_l = (1-t_loc)*bqc_h0+t_loc*bqc_l0;
    
    if abs(f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)-beta0)
        bqc_temp = bqc_l;
    else
        bqc_temp = bqc_h;
    end
    temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);

        if qc<q2
        q1 = qc;
    bq1 = bqc_temp;
    bq2 = f_pc1(q2,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    else
        q2 = qc;
        bq2 = bqc_temp;
        bq1 = f_pc1(q1,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    end
    
%    Lp = [Lp(:); p2; qc];
%    Lb = [Lb(:); b2; bqc_temp];

    
    end
else

    qc = intersection(p1,p2,p3,p4,b1,b2,b3,b4);
    if (qc<(1-lc)*p2+lc*p3)||(qc>(1-lc)*p3+lc*p2)
        qc = (p2+p3)/2;
    end
    
    bqc_h = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
    bqc_l = env2(qc,p1,p2,p3,p4,b1,b2,b3,b4);
    
    if bqc_l>bqc_h-0.5
        bqc_l = bqc_h-0.5;
    end
    
    temp = f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb);
    
    if temp<beta0
    while abs(temp-beta0)>eps
       t_loc = 0; 
       
    
    while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqc_h+t_loc*bqc_l,b3,Lp,Lb)<beta0
        t_loc = t_loc+delta;
    end
    
    bqc_l0 = bqc_l;
    bqc_h0 = bqc_h;
    
    bqc_h = (1-(t_loc-delta))*bqc_h0+(t_loc-delta)*bqc_l0;
    bqc_l = (1-t_loc)*bqc_h0+t_loc*bqc_l0;
    
    if abs(f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)-beta0)
        bqc_temp = bqc_l;
    else
        bqc_temp = bqc_h;
    end
    temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);

        if qc<q2
        q1 = qc;
    bq1 = bqc_temp;
    bq2 = f_pc1(q2,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    else
        q2 = qc;
        bq2 = bqc_temp;
        bq1 = f_pc1(q1,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    end
    
%    Lp = [Lp(:); p2; qc];
%    Lb = [Lb(:); b2; bqc_temp];
    else
        
    bqc_l = bqc_h;
    bqc_h = bqc_l+0.5;
    while abs(temp-beta0)>eps
       t_loc = 0; 
       

    
    
    
    while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqc_l+t_loc*bqc_h,b3,Lp,Lb)>beta0
        t_loc = t_loc+delta;
    end
    
    bqc_l0 = bqc_l;
    bqc_h0 = bqc_h;
    
    bqc_l = (1-(t_loc-delta))*bqc_l0+(t_loc-delta)*bqc_h0;
    bqc_h = (1-t_loc)*bqc_l0+t_loc*bqc_h0;
    
    if abs(f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)-beta0)
        bqc_temp = bqc_l;
    else
        bqc_temp = bqc_h;
    end
    temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);

        if qc<q2
        q1 = qc;
    bq1 = bqc_temp;
    bq2 = f_pc1(q2,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    else
        q2 = qc;
        bq2 = bqc_temp;
        bq1 = f_pc1(q1,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    end
    
%    Lp = [Lp(:); p2; qc];
%    Lb = [Lb(:); b2; bqc_temp];
    end
end

else
    bq2_min = env2(q2,p1,p2,p3,p4,b1,b2,b3,b4);
    
if (integral(@(p)(1./((1-p).*env1(p,p1,p2,p3,p4,b1,b2,b3,b4))),p2,p3)>=beta0)&&(integral(@(p)(1./((1-p).*env2(p,p1,p2,p3,p4,b1,b2,b3,b4))),p2,p3)<=beta0)


if integral(@(p)(1./((1-p).*interp1([p2; q1; q2; p3],[b2; BQ1(bq2_min,b2,b3,1/2); bq2_min; b3],p,'linear','extrap'))),p2,p3)>beta0
    

    bq2_l = interp1([p2; p3],[b2; b3],q2,'linear','extrap');
    bq2_h = env2(q2,p1,p2,p3,p4,b1,b2,b3,b4);
    
    if bq2_h<bq2_l+0.5
        bq2_h=bq2_l+0.5;
    end
    
%    pause
    
%    temp = f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_l,b2,b3,1/2),bq2_l,b3,Lp,Lb)
%    beta0
    
%    pause
    
    
%    pp_draw = linspace(p2,p3,20);
    
    while abs(temp-beta0)>eps
       t_loc = 0;
       
    while f_beta2(p2,q1,q2,p3,b2,BQ1((1-t_loc)*bq2_l+t_loc*bq2_h,b2,b3,1/2),(1-t_loc)*bq2_l+t_loc*bq2_h,b3,Lp,Lb)>beta0
        t_loc = t_loc+delta;
%        (1-t_loc)*bq2_l+t_loc*bq2_h
%        BQ1((1-t_loc)*bq2_l+t_loc*bq2_h,b2,b3,1/2)
%        f_beta2(p2,q1,q2,p3,b2,BQ1((1-t_loc)*bq2_l+t_loc*bq2_h,b2,b3,1/2),(1-t_loc)*bq2_l+t_loc*bq2_h,b3,Lp,Lb)
%        beta0
%        
%        f_pc2(pp_draw,p2,q1,q2,p3,b2,BQ1((1-t_loc)*bq2_l+t_loc*bq2_h,b2,b3,1/2),(1-t_loc)*bq2_l+t_loc*bq2_h,b3,Lp,Lb)
%        pause
%        pause
%        f_beta2(p2,q1,q2,p3,b2,BQ1((1-t_loc)*bq2_l+t_loc*bq2_h,b2,b3,1/2),(1-t_loc)*bq2_l+t_loc*bq2_h,b3,Lp,Lb)
%        beta0
%        pause
    end
    
    bq2_l0 = bq2_l;
    bq2_h0 = bq2_h;
    
    bq2_l = (1-(t_loc-delta))*bq2_l0+(t_loc-delta)*bq2_h0;
    bq2_h = (1-t_loc)*bq2_l0+t_loc*bq2_h0;
%    pause
    
    if abs(f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_l,b2,b3,1/2),bq2_l,b3,Lp,Lb)-beta0)<abs(f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_h,b2,b3,1/2),bq2_h,b3,Lp,Lb)-beta0)
        bq2_temp = bq2_l;
    else
        bq2_temp = bq2_h;
    end
    temp = f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_temp,b2,b3,1/2),bq2_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc2(PP((PP>=p2)&(PP<=p3)),p2,q1,q2,p3,b2,BQ1(bq2_temp,b2,b3,1/2),bq2_temp,b3,Lp,Lb);
    
    Lp = [Lp(:); p2; q1; q2];
    Lb = [Lb(:); b2; BQ1(bq2_temp,b2,b3,1/2); bq2_temp];
    
    bq2 = bq2_temp;
    bq1 = BQ1(bq2_temp,b2,b3,1/2);
    
%    end
    
elseif integral(@(p)(1./((1-p).*interp1([p2; q1; q2; p3],[b2; BQ1(bq2_min,b2,b3,3/4); bq2_min; b3],p,'linear','extrap'))),p2,p3)>=beta0

    bq2_h = interp1([p2; p3],[b2; b3],q2,'linear','extrap');
    bq2_l = env2(q2,p1,p2,p3,p4,b1,b2,b3,b4);
    
    
    temp = f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_l,b2,b3,3/4),bq2_l,b3,Lp,Lb);
    
    while abs(temp-beta0)>eps
       t_loc = 0;
       
    while f_beta2(p2,q1,q2,p3,b2,BQ1((1-t_loc)*bq2_l+t_loc*bq2_h,b2,b3,3/4),(1-t_loc)*bq2_l+t_loc*bq2_h,b3,Lp,Lb)<beta0
        t_loc = t_loc+delta;
    end
    
    bq2_l0 = bq2_l;
    bq2_h0 = bq2_h;
    
    bq2_h = (1-(t_loc-delta))*bq2_l0+(t_loc-delta)*bq2_h0;
    bq2_l = (1-t_loc)*bq2_l0+t_loc*bq2_h0;
    
    if abs(f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_l,b2,b3,3/4),bq2_l,b3,Lp,Lb)-beta0)<abs(f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_h,b2,b3,3/4),bq2_h,b3,Lp,Lb)-beta0)
        bq2_temp = bq2_l;
    else
        bq2_temp = bq2_h;
    end
    temp = f_beta2(p2,q1,q2,p3,b2,BQ1(bq2_temp,b2,b3,3/4),bq2_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc2(PP((PP>=p2)&(PP<=p3)),p2,q1,q2,p3,b2,BQ1(bq2_temp,b2,b3,3/4),bq2_temp,b3,Lp,Lb);
    
    Lp = [Lp(:); p2; q1; q2];
    Lb = [Lb(:); b2; BQ1(bq2_temp,b2,b3,3/4);bq2_temp];
    
    bq1 = BQ1(bq2_temp,b2,b3,3/4);
    bq2 = bq2_temp;
    
else
        qc = intersection(p1,p2,p3,p4,b1,b2,b3,b4);
    if (qc<(1-lc)*p2+lc*p3)||(qc>(1-lc)*p3+lc*p2)
        qc = (p2+p3)/2;
    end
    
    bqc_temp = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
    temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    
    if temp>=beta0
    
    bqc_l = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
    bqc_h = env2(qc,p1,p2,p3,p4,b1,b2,b3,b4);
    
    if bqc_h<bqc_l+0.5
        bqc_h = bqc_l+0.5;
    end
    
        
    temp = f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb);
    
    while abs(temp-beta0)>eps
       t_loc = 0; 
       
    while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqc_l+t_loc*bqc_h,b3,Lp,Lb)>beta0
        t_loc = t_loc+delta;
    end
    
    bqc_l0 = bqc_l;
    bqc_h0 = bqc_h;
    
    bqc_l = (1-(t_loc-delta))*bqc_l0+(t_loc-delta)*bqc_h0;
    bqc_h = (1-t_loc)*bqc_l0+t_loc*bqc_h0;
    
    if abs(f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)-beta0)
        bqc_temp = bqc_l;
    else
        bqc_temp = bqc_h;
    end
    temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    

    if qc<q2
        q1 = qc;
    bq1 = bqc_temp;
    bq2 = f_pc1(q2,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    else
        q2 = qc;
        bq2 = bqc_temp;
        bq1 = f_pc1(q1,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    end
    
%    Lp = [Lp(:); p2; qc];
%    Lb = [Lb(:); b2; bqc_temp];
    
    else %temp<beta0
    bqc_h = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
    bqc_l = bqc_h-1;
    

    
         
    temp = f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb);
    
    while abs(temp-beta0)>eps
       t_loc = 0; 
       
    while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqc_h+t_loc*bqc_l,b3,Lp,Lb)<beta0
        t_loc = t_loc+delta;
    end
    
    bqc_l0 = bqc_l;
    bqc_h0 = bqc_h;
    
    bqc_h = (1-(t_loc-delta))*bqc_h0+(t_loc-delta)*bqc_l0;
    bqc_l = (1-t_loc)*bqc_h0+t_loc*bqc_l0;
    
    if abs(f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)-beta0)
        bqc_temp = bqc_l;
    else
        bqc_temp = bqc_h;
    end
    temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    

    if qc<q2
        q1 = qc;
    bq1 = bqc_temp;
    bq2 = f_pc1(q2,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    else
        q2 = qc;
        bq2 = bqc_temp;
        bq1 = f_pc1(q1,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    end
    
%    Lp = [Lp(:); p2; qc];
%    Lb = [Lb(:); b2; bqc_temp];
    end
    
    end
else
    qc = intersection(p1,p2,p3,p4,b1,b2,b3,b4);
    
    if (qc<(1-lc)*p2+lc*p3)||(qc>(1-lc)*p3+lc*p2)
        qc = (p2+p3)/2;
    end
    
    
    bqc_temp = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
    temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    
    if temp>=beta0
    
    bqc_l = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
    bqc_h = env2(qc,p1,p2,p3,p4,b1,b2,b3,b4);
    
    if bqc_h<bqc_l+0.5
        bqc_h = bqc_l+0.5;
    end
    
        
    temp = f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb);
    
    while abs(temp-beta0)>eps
       t_loc = 0; 
       
    while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqc_l+t_loc*bqc_h,b3,Lp,Lb)>beta0
        t_loc = t_loc+delta;
    end
    
    bqc_l0 = bqc_l;
    bqc_h0 = bqc_h;
    
    bqc_l = (1-(t_loc-delta))*bqc_l0+(t_loc-delta)*bqc_h0;
    bqc_h = (1-t_loc)*bqc_l0+t_loc*bqc_h0;
    
    if abs(f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)-beta0)
        bqc_temp = bqc_l;
    else
        bqc_temp = bqc_h;
    end
    temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    

    if qc<q2
        q1 = qc;
    bq1 = bqc_temp;
    bq2 = f_pc1(q2,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    else
        q2 = qc;
        bq2 = bqc_temp;
        bq1 = f_pc1(q1,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    end
    
%    Lp = [Lp(:); p2; qc];
%    Lb = [Lb(:); b2; bqc_temp];
    
    else %temp<beta0
    bqc_h = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
    bqc_l = bqc_h-1;
    

        
    temp = f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb);
    
    while abs(temp-beta0)>eps
       t_loc = 0; 
       
    while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqc_h+t_loc*bqc_l,b3,Lp,Lb)<beta0
        t_loc = t_loc+delta;
    end
    
    bqc_l0 = bqc_l;
    bqc_h0 = bqc_h;
    
    bqc_h = (1-(t_loc-delta))*bqc_h0+(t_loc-delta)*bqc_l0;
    bqc_l = (1-t_loc)*bqc_h0+t_loc*bqc_l0;
    
    if abs(f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)-beta0)
        bqc_temp = bqc_l;
    else
        bqc_temp = bqc_h;
    end
    temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    
    end
    
    BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    

    if qc<q2
        q1 = qc;
    bq1 = bqc_temp;
    bq2 = f_pc1(q2,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    else
        q2 = qc;
        bq2 = bqc_temp;
        bq1 = f_pc1(q1,p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
    end
    
%    Lp = [Lp(:); p2; qc];
%    Lb = [Lb(:); b2; bqc_temp];
    end
    
end


end
end












%% 3. Approximation of threshold T-1
% 
% p1 = pp(end-2);
% p2 = pp(end-1);
% p3 = pp(end);
% 
% b1 = bb(end-2);
% b2 = bb(end-1);
% b3 = bb(end);
% 
% q1 = (2/3)*p2+(1/3)*p3;
% q2 = (1/3)*p2+(2/3)*p3;
% 
% qc = (9/10)*p3+(1/10)*p2;
% 
% qm = (1/2)*p2+(1/2)*p3;
% 
% %f_pc2_end = @(p,p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb)(interp1([Lp(:); p2;q1;q2;p3],[Lb(:); b2;bq1;bq2;b3],p,'pchip','extrap'));
%     
% %f_beta2_end = @(p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb)(quadgk(@(q)(1./((1-q).*f_pc2(q,p2,q1,q2,p3,b2,bq1,bq2,b3,Lp,Lb))),p2,p3,'MaxIntervalCount',10000));
% 
% 
% beta0 = beta(end)
% 
% temp = integral(@(p)(1./((1-p).*interp1([p2; p3],[b2; b3],p,'linear','extrap'))),p2,p3);
% if temp>beta0
%     if (b2-b1)/(p2-p1)>(b3-b2)/(p3-p2)
%         bq1_h = interp1([p1; p2],[b1; b2],q1,'linear','extrap');
%         bq2_h = interp1([p1; p2],[b1; b2],q2,'linear','extrap');
%         
%         bq1_l = interp1([p2; p3],[b2; b3],q1,'linear','extrap');
%         bq2_l = interp1([p2; p3],[b2; b3],q2,'linear','extrap');
%         
%         bqc_h = interp1([p1; p2],[b1; b2],qc,'linear','extrap');
%         bqc_l = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
%         
%         bqm_h = interp1([p1; p2],[b1; b2],qm,'linear','extrap');
%         bqm_l = interp1([p2; p3],[b2; b3],qm,'linear','extrap');
%         
%         if integral(@(p)(1./((1-p).*interp1([p2; q1; q2; p3],[b2; bq1_h; bq2_h; b3],p,'pchip','extrap'))),p2,p3)<beta0
%   
%     
%     temp = f_beta2(p2,q1,q2,p3,b2,bq1_l,bq2_l,b3,Lp,Lb);
%     
%     1
%     while abs(temp-beta0)>eps
%        t_loc = 0; 
%        
%     while f_beta2(p2,q1,q2,p3,b2,(1-t_loc)*bq1_l+t_loc*bq1_h,(1-t_loc)*bq2_l+t_loc*bq2_h,b3,Lp,Lb)>beta0
%         t_loc = t_loc+delta;
%     end
%     bq1_l0 = bq1_l;
%     bq1_h0 = bq1_h;
%         
%     bq2_l0 = bq2_l;
%     bq2_h0 = bq2_h;
% 
%     bq1_l = (1-(t_loc-delta))*bq1_l0+(t_loc-delta)*bq1_h0;
%     bq1_h = (1-t_loc)*bq1_l0+t_loc*bq1_h0;
%     
%     bq2_l = (1-(t_loc-delta))*bq2_l0+(t_loc-delta)*bq2_h0;
%     bq2_h = (1-t_loc)*bq2_l0+t_loc*bq2_h0;
%     
%     if abs(f_beta2(p2,q1,q2,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)-beta0)<abs(f_beta2(p2,q1,q2,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)-beta0)
%         bq1_temp = bq1_l;
%         bq2_temp = bq2_l;
%     else
%         bq1_temp = bq1_h;
%         bq2_temp = bq2_h;
%     end
%     temp = f_beta2(p2,q1,q2,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
%     
%     end
%     
%     BB((PP>=p2)&(PP<=p3)) = f_pc2(PP((PP>=p2)&(PP<=p3)),p2,q1,q2,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
%     
%     Lp = [Lp(:); p2; q1; q2; p3];
%     Lb = [Lb(:); b2; bq1_temp; bq2_temp; p3];
% 
%     
%             
%         elseif f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)>beta0
%   
%     
%     temp = f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb);
%     
%     2
%     while abs(temp-beta0)>eps
%        t_loc = 0; 
%        
%     while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqc_l+t_loc*bqc_h,b3,Lp,Lb)>beta0
%         t_loc = t_loc+delta;
%     end
%     bqc_l0 = bqc_l;
%     bqc_h0 = bqc_h;
% 
%     bqc_l = (1-(t_loc-delta))*bqc_l0+(t_loc-delta)*bqc_h0;
%     bqc_h = (1-t_loc)*bqc_l0+t_loc*bqc_h0;
%     
% 
%     if abs(f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)-beta0)
%         bqc_temp = bqc_l;
%     else
%         bqc_temp = bqc_h;
%     end
%     temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
%     
%     end
%     
%     BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
%     
%     Lp = [Lp(:); p2; qc; p3];
%     Lb = [Lb(:); b2; bqc_temp; p3];
%             
%         else
%             
%             
%             
%             
%             
%     temp = f_beta1(p2,qm,p3,b2,bqm_l,b3,Lp,Lb);
%     
%     3
%     while abs(temp-beta0)>eps
%        t_loc = 0; 
%        
%     while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqm_l+t_loc*bqm_h,b3,Lp,Lb)>beta0
%         t_loc = t_loc+delta;
%     end
%     bqm_l0 = bqm_l;
%     bqm_h0 = bqm_h;
% 
%     bqm_l = (1-(t_loc-delta))*bqm_l0+(t_loc-delta)*bqm_h0;
%     bqm_h = (1-t_loc)*bqm_l0+t_loc*bqm_h0;
%     
% 
%     if abs(f_beta1(p2,qm,p3,b2,bqm_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qm,p3,b2,bqm_h,b3,Lp,Lb)-beta0)
%         bqm_temp = bqm_l;
%     else
%         bqm_temp = bqm_h;
%     end
%     temp = f_beta1(p2,qm,p3,b2,bqm_temp,b3,Lp,Lb);
%     
%     end
%     
%     BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qm,p3,b2,bqm_temp,b3,Lp,Lb);
%     
%     Lp = [Lp(:); p2; qm; p3];
%     Lb = [Lb(:); b2; bqm_temp; p3];
%     
%         
%         end
%     else 
%         
%                         bq1_h = interp1([p1; p2],[b1; b2],q1,'linear','extrap');
%         bq2_h = interp1([p1; p2],[b1; b2],q2,'linear','extrap');
%         
%         bq1_l = interp1([p2; p3],[b2; b3],q1,'linear','extrap');
%         bq2_l = interp1([p2; p3],[b2; b3],q2,'linear','extrap');
%         
%             temp = f_beta2(p2,q1,q2,p3,b2,bq1_l,bq2_l,b3,Lp,Lb);
%     
%             4
%             
%     while abs(temp-beta0)>eps
%        t_loc = 0; 
%        
%     while f_beta2(p2,q1,q2,p3,b2,(1-t_loc)*bq1_l+t_loc*bq1_h,(1-t_loc)*bq2_l+t_loc*bq2_h,b3,Lp,Lb)>beta0
%         t_loc = t_loc+delta;
%     end
%     bq1_l0 = bq1_l;
%     bq1_h0 = bq1_h;
%         
%     bq2_l0 = bq2_l;
%     bq2_h0 = bq2_h;
% 
%     bq1_l = (1-(t_loc-delta))*bq1_l0+(t_loc-delta)*bq1_h0;
%     bq1_h = (1-t_loc)*bq1_l0+t_loc*bq1_h0;
%     
%     bq2_l = (1-(t_loc-delta))*bq2_l0+(t_loc-delta)*bq2_h0;
%     bq2_h = (1-t_loc)*bq2_l0+t_loc*bq2_h0;
%     
%     if abs(f_beta2(p2,q1,q2,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)-beta0)<abs(f_beta2(p2,q1,q2,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)-beta0)
%         bq1_temp = bq1_l;
%         bq2_temp = bq2_l;
%     else
%         bq1_temp = bq1_h;
%         bq2_temp = bq2_h;
%     end
%     temp = f_beta2(p2,q1,q2,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
%     
%     end
%     
%     BB((PP>=p2)&(PP<=p3)) = f_pc2(PP((PP>=p2)&(PP<=p3)),p2,q1,q2,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
%     
%     Lp = [Lp(:); p2; q1; q2; p3];
%     Lb = [Lb(:); b2; bq1_temp; bq2_temp; p3];
% 
%         
%     end
% else
%     if (b2-b1)/(p2-p1)>(b3-b2)/(p3-p2)
%         bq1_l = interp1([p1; p2],[b1; b2],q1,'linear','extrap');
%         bq2_l = interp1([p1; p2],[b1; b2],q2,'linear','extrap');
%         
%         bq1_h = interp1([p2; p3],[b2; b3],q1,'linear','extrap');
%         bq2_h = interp1([p2; p3],[b2; b3],q2,'linear','extrap');
%         
%         bqc_l = interp1([p1; p2],[b1; b2],qc,'linear','extrap');
%         bqc_h = interp1([p2; p3],[b2; b3],qc,'linear','extrap');
%         
%         bqm_l = interp1([p1; p2],[b1; b2],qm,'linear','extrap');
%         bqm_h = interp1([p2; p3],[b2; b3],qm,'linear','extrap');
%         
%         if integral(@(p)(1./((1-p).*interp1([p2; q1; q2; p3],[b2; bq1_l; bq2_l; b3],p,'pchip','extrap'))),p2,p3)>beta0
%   
%     
%     temp = f_beta2(p2,q1,q2,p3,b2,bq1_h,bq2_h,b3,Lp,Lb);
%     
%     5
%     while abs(temp-beta0)>eps
%        t_loc = 0; 
%        
%     while f_beta2(p2,q1,q2,p3,b2,(1-t_loc)*bq1_h+t_loc*bq1_l,(1-t_loc)*bq2_h+t_loc*bq2_l,b3,Lp,Lb)>beta0
%         t_loc = t_loc+delta;
%     end
%     bq1_l0 = bq1_l;
%     bq1_h0 = bq1_h;
%         
%     bq2_l0 = bq2_l;
%     bq2_h0 = bq2_h;
% 
%     bq1_h = (1-(t_loc-delta))*bq1_h0+(t_loc-delta)*bq1_l0;
%     bq1_l = (1-t_loc)*bq1_h0+t_loc*bq1_l0;
%     
%     bq2_h = (1-(t_loc-delta))*bq2_h0+(t_loc-delta)*bq2_l0;
%     bq2_l = (1-t_loc)*bq2_h0+t_loc*bq2_l0;
%     
%     if abs(f_beta2(p2,q1,q2,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)-beta0)<abs(f_beta2(p2,q1,q2,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)-beta0)
%         bq1_temp = bq1_l;
%         bq2_temp = bq2_l;
%     else
%         bq1_temp = bq1_h;
%         bq2_temp = bq2_h;
%     end
%     temp = f_beta2(p2,q1,q2,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
%     
%     end
%     
%     BB((PP>=p2)&(PP<=p3)) = f_pc2(PP((PP>=p2)&(PP<=p3)),p2,q1,q2,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
%     
%     Lp = [Lp(:); p2; q1; q2; p3];
%     Lb = [Lb(:); b2; bq1_temp; bq2_temp; p3];
% 
%     
%             
%         elseif f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)<beta0
%     
%             if bqc_h<bqc_l
%                 bqc_h=bqc_h+0.5;
%             end
% 
%     
%     temp = f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb);
%     
%     6
%     while abs(temp-beta0)>eps
%        t_loc = 0; 
%        
%     while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqc_h+t_loc*bqc_l,b3,Lp,Lb)<beta0
%         t_loc = t_loc+delta;
%     end
%     bqc_l0 = bqc_l;
%     bqc_h0 = bqc_h;
% 
%     bqc_h = (1-(t_loc-delta))*bqc_h0+(t_loc-delta)*bqc_l0;
%     bqc_l = (1-t_loc)*bqc_h0+t_loc*bqc_l0;
%     
% 
%     if abs(f_beta1(p2,qc,p3,b2,bqc_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qc,p3,b2,bqc_h,b3,Lp,Lb)-beta0)
%         bqc_temp = bqc_l;
%     else
%         bqc_temp = bqc_h;
%     end
%     temp = f_beta1(p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
%     
%     end
%     
%     BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qc,p3,b2,bqc_temp,b3,Lp,Lb);
%     
%     Lp = [Lp(:); p2; qc; p3];
%     Lb = [Lb(:); b2; bqc_temp; p3];
%             
%         else
%             
%             
%             
%             
%             
%     temp = f_beta1(p2,qm,p3,b2,bqm_h,b3,Lp,Lb);
%     7
%     while abs(temp-beta0)>eps
%        t_loc = 0; 
%        
%     while f_beta1(p2,qc,p3,b2,(1-t_loc)*bqm_h+t_loc*bqm_l,b3,Lp,Lb)>beta0
%         t_loc = t_loc+delta;
%     end
%     bqm_l0 = bqm_l;
%     bqm_h0 = bqm_h;
% 
%     bqm_h = (1-(t_loc-delta))*bqm_h0+(t_loc-delta)*bqm_l0;
%     bqm_l = (1-t_loc)*bqm_h0+t_loc*bqm_l0;
%     
% 
%     if abs(f_beta1(p2,qm,p3,b2,bqm_l,b3,Lp,Lb)-beta0)<abs(f_beta1(p2,qm,p3,b2,bqm_h,b3,Lp,Lb)-beta0)
%         bqm_temp = bqm_l;
%     else
%         bqm_temp = bqm_h;
%     end
%     temp = f_beta1(p2,qm,p3,b2,bqm_temp,b3,Lp,Lb);
%     
%     end
%     
%     BB((PP>=p2)&(PP<=p3)) = f_pc1(PP((PP>=p2)&(PP<=p3)),p2,qm,p3,b2,bqm_temp,b3,Lp,Lb);
%     
%     Lp = [Lp(:); p2; qm; p3];
%     Lb = [Lb(:); b2; bqm_temp; p3];
%     
%         
%         end
%     else 
%         
%         bq1_l = interp1([p1; p2],[b1; b2],q1,'linear','extrap');
%         bq2_l = interp1([p1; p2],[b1; b2],q2,'linear','extrap');
%         
%         bq1_h = interp1([p2; p3],[b2; b3],q1,'linear','extrap');
%         bq2_h = interp1([p2; p3],[b2; b3],q2,'linear','extrap');
%         
%             temp = f_beta2(p2,q1,q2,p3,b2,bq1_h,bq2_l,b3,Lp,Lb);
%     8
%     while abs(temp-beta0)>eps
%        t_loc = 0; 
%        
%     while f_beta2(p2,q1,q2,p3,b2,(1-t_loc)*bq1_h+t_loc*bq1_l,(1-t_loc)*bq2_h+t_loc*bq2_l,b3,Lp,Lb)>beta0
%         t_loc = t_loc+delta;
%     end
%     bq1_l0 = bq1_l;
%     bq1_h0 = bq1_h;
%         
%     bq2_l0 = bq2_l;
%     bq2_h0 = bq2_h;
% 
%     bq1_h = (1-(t_loc-delta))*bq1_h0+(t_loc-delta)*bq1_l0;
%     bq1_l = (1-t_loc)*bq1_h0+t_loc*bq1_l0;
%     
%     bq2_h = (1-(t_loc-delta))*bq2_h0+(t_loc-delta)*bq2_l0;
%     bq2_l = (1-t_loc)*bq2_h0+t_loc*bq2_l0;
%     
%     if abs(f_beta2(p2,q1,q2,p3,b2,bq1_l,bq2_l,b3,Lp,Lb)-beta0)<abs(f_beta2(p2,q1,q2,p3,b2,bq1_h,bq2_h,b3,Lp,Lb)-beta0)
%         bq1_temp = bq1_l;
%         bq2_temp = bq2_l;
%     else
%         bq1_temp = bq1_h;
%         bq2_temp = bq2_h;
%     end
%     temp = f_beta2(p2,q1,q2,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
%     
%     end
%     
%     BB((PP>=p2)&(PP<=p3)) = f_pc2(PP((PP>=p2)&(PP<=p3)),p2,q1,q2,p3,b2,bq1_temp,bq2_temp,b3,Lp,Lb);
%     
%     Lp = [Lp(:); p2; q1; q2; p3];
%     Lb = [Lb(:); b2; bq1_temp; bq2_temp; p3];
% 
%     end
% end



%% 4. Approximation of the upper part

eta = (pp(end-1)-pp(end-2))/4;
b0 = interp1(Lp,Lb,pp(end-1)-eta,'pchip','extrap');

BB(PP>=pp(end-1)) = interp1([pp(end-1)-eta; pp(end-1)],[b0; bb(end-1)],PP(PP>=pp(end-1)),'linear','extrap');
%bmin = (min(bb(1:end-1))+1)/2;
BB = max(BB,ones(size(BB)));

%% 5. Approximation of the lower part

beta1 = log(((1-pp(1)).*thr(1).*bb(1))./((1-pp(2)).*thr(2).*bb(2)));

%m1 = (pp(1)+pp(2))/2;
m1 = 0.999*pp(1)+0.001*pp(2);
m2 = 0.6*pp(1)+0.4*pp(2);

pp_low = [pp(1); m1; m2; Lp(:)];

f_low = @(p,bm1,bm2)(interp1(pp_low,[bb(1); bm1; bm2; Lb(:)],p,'pchip','extrap'));

f_beta_low = @(bm1,bm2)(quadgk(@(q)(1./((1-q).*f_low(q,bm1,bm2))),pp(1),pp(2),'MaxIntervalCount',10000));

bm1_h = interp1(pp(1:3),bb(1:3),m1,'linear','extrap');
bm1_l = interp1(pp(2:3),bb(2:3),m1,'linear','extrap');

bm2_h = interp1(pp(1:3),bb(1:3),m2,'linear','extrap');
bm2_l = interp1(pp(2:3),bb(2:3),m2,'linear','extrap');
%bm1_pchip = interp1(pp(1:3),bb(1:3),m1,'pchip','extrap');
%bm2_pchip = interp1(pp(1:3),bb(1:3),m1,'pchip','extrap');

%beta1
%f_beta_low(bm1_pchip,bm2_pchip)

%if beta1>f_beta_low(bm_pchip)
%    g = @(t)(f_beta((1-t)*bm_pchip+t*bm_l))


temp = f_beta_low(bm1_l,bm2_l);
    
if f_beta_low(bm1_l,bm2_l)>beta1
    bm1_h = bm1_l+1;
    bm2_h = bm2_l+1;
    
    while abs(temp-beta1)>eps
       t_loc = 0; 
       
    while f_beta_low((1-t_loc)*bm1_l+t_loc*bm1_h,(1-t_loc/5)*bm2_l+(t_loc/5)*bm2_h)>beta1
        t_loc = t_loc+delta;
    end
    
    bm1_l0 = bm1_l;
    bm1_h0 = bm1_h;
    
    bm2_l0 = bm2_l;
    bm2_h0 = bm2_h;

    bm1_l = (1-(t_loc-delta))*bm1_l0+(t_loc-delta)*bm1_h0;
    bm1_h = (1-t_loc)*bm1_l0+t_loc*bm1_h0;
    
    bm2_l = (1-(t_loc-delta)/5)*bm2_l0+((t_loc-delta)/5)*bm2_h0;
    bm2_h = (1-t_loc/5)*bm2_l0+(t_loc/5)*bm2_h0;
    
    if abs(f_beta_low(bm1_l,bm2_l)-beta1)<abs(f_beta_low(bm1_h,bm2_h)-beta1)
        bm1_temp = bm1_l;
        bm2_temp = bm2_l;
    else
        bm1_temp = bm2_h;
        bm2_temp = bm2_h;
    end
    temp = f_beta_low(bm1_temp,bm2_temp);
    
    end


%     g = @(t,ll,hh)(f_beta_low((1-t)*hh+t*ll));
%     
%     delta = 0.01;
%     ll_temp = bm_l;
%     hh_temp = bm_pchip;
%     bm_temp = bm_pchip;
%     
%     while abs(f_beta_low(bm_temp)-beta1)>eps
%     t_loc=0;
%     
%     while g(t_loc,ll_temp,hh_temp)<beta1
%         t_loc = t_loc+delta;
%     end
%     ll_temp0 = ll_temp;
%     hh_temp0 = hh_temp;
%     
%     hh_temp = (1-(t_loc-delta))*hh_temp0+(t_loc-delta)*ll_temp0;
%     ll_temp = (1-t_loc)*hh_temp0+t_loc*ll_temp0;
% %    delta = delta/10
%     if abs(f_beta_low(ll_temp)-beta1)<abs(f_beta_low(hh_temp)-beta1)
%         bm_temp = ll_temp;
%     else
%         bm_temp = hh_temp;
%     end
%     end
    

%end

%bm1 = bm_temp;

else
    bm1_h = bm1_l;
    bm2_h = bm2_l;
    bm1_l = bm1_h-1;
    bm2_l = bm2_h-1;

    while abs(temp-beta1)>eps
        t_loc = 0;
        

    while f_beta_low((1-t_loc/5)*bm1_h+(t_loc/5)*bm1_l,(1-t_loc)*bm2_h+t_loc*bm2_l)<beta1
        t_loc = t_loc+delta;
    end
    
    bm1_l0 = bm1_l;
    bm1_h0 = bm1_h;
    
    bm2_l0 = bm2_l;
    bm2_h0 = bm2_h;

    bm1_h = (1-(t_loc-delta)/5)*bm1_h0+((t_loc-delta)/5)*bm1_l0;
    bm1_l = (1-t_loc/5)*bm1_h0+(t_loc/5)*bm1_l0;
    
    bm2_h = (1-(t_loc-delta))*bm2_h0+(t_loc-delta)*bm2_l0;
    bm2_l = (1-t_loc)*bm2_h0+t_loc*bm2_l0;
    
    if abs(f_beta_low(bm1_l,bm2_l)-beta1)<abs(f_beta_low(bm1_h,bm2_h)-beta1)
        bm1_temp = bm1_l;
        bm2_temp = bm2_l;
    else
        bm1_temp = bm2_h;
        bm2_temp = bm2_h;
    end
    temp = f_beta_low(bm1_temp,bm2_temp);
    
    end
    
end

    BB(PP<=pp(2)) = f_low(PP(PP<=pp(2)),bm1_temp,bm2_temp);




end
