%% Estimation of the lower part of the distribution using a Singh-Maddala functional form
% 
% Steps of the program:
%   * 1. Computes the density and its derivative at p1
%   * 2. Defines the CDF, the PDF, its derivative and the quantile function of the Singh-Maddala distribution
%   * 3. Converts these symbolic functions to Matlab functions
%   * 4. Minimizes fmin and adjusts p0
%   * 5. Estimates Q_low with optimal parameters
%
function [Q_low] = lower_part(q,q_pr,q_sec,p1,p0_max,mu,P_low,p0)

%% 1. Computes the density and its derivative at p1
f1 = 1/q_pr;
f1_pr = -(f1)^3*q_sec;

%% 2. Defines the CDF, the PDF, its derivative and the quantile function of the Singh-Maddala distribution
syms p y c k s z

F_sym = z+1-(1+(s*y).^c).^(-k); % CDF
f_sym = diff(F_sym,y); % PDF
f_sym_pr = diff(f_sym,y); % derivative of the PDF
Q_sym = (1/s)*((1-p).^(-1/k)-1).^(1/c); % quantile function

%% 3. Converts these symbolic functions to Matlab functions

F = subs(F_sym,y,q); % evaluates the CDF at q=Q(p1)
f = subs(f_sym,y,q); % evaluates the PDF at q
f_pr = subs(f_sym_pr,y,q); % evaluates the derivative of the PDF at q

F_func = matlabFunction(F); % converts F
f_func = matlabFunction(f); % converts f
f_pr_func = matlabFunction(f_pr); % converts f_pr
Q_func = matlabFunction(Q_sym); % converts Q

%% 4. Defines the functional to be minimized
fmin = @(c,k,s,z)(1000*(F_func(c,k,s,z)-p1).^2+20*(f_func(c,k,s)-f1).^2+(f_pr_func(c,k,s)-f1_pr).^2);
% fmin is minimal for parameters c, k and s such that F=p1, f=f1 and
% f_pr=f1_pr

if isnan(p0)
%% 5. Initialization
options = optimoptions('fmincon','Display','off');
param = fmincon(@(x)fmin(x(1),x(2),x(3),x(4)),[2,1,5,0],[],[],[],[],[0,0,0,0],[20,20,20,p0_max],[],options);

p0 = param(4);

% criterion for optimality
p1_temp = F_func(param(1),param(2),param(3),p0);
f1_temp = f_func(param(1),param(2),param(3));
f1_pr_temp = f_pr_func(param(1),param(2),param(3));
m_temp = (1/F_func(param(1),param(2),param(3),p0))*integral(@(p)(Q_func(param(1),param(2),p,param(3))),0,F_func(param(1),param(2),param(3),p0)-p0);
        
c_temp = abs(p1_temp-p1)/p1+abs(f1_temp-f1)/f1+abs((f1_pr_temp-f1_pr)/f1_pr)+100*abs(m_temp-mu)/mu;
c_min = c_temp;

Q_low = zeros(size(P_low,1),1);
Q_low((F_func(param(1),param(2),param(3),p0)/p1)*P_low>p0) = Q_func(param(1),param(2),(F_func(param(1),param(2),param(3),p0)/p1)*P_low((F_func(param(1),param(2),param(3),p0)/p1)*P_low>p0)-p0,param(3));

    
%% 6. Minimizes fmin and adjusts p0
iter = 0; % number of iterations

while (c_temp>0.01)&&(iter<20)
iter = iter+1;

p0_old = p0;

p0 = p0-F_func(param(1),param(2),param(3),p0)*(mu-m_temp)/Q_func(param(1),param(2),F_func(param(1),param(2),param(3),p0)-p0,param(3));
p0 = min(max(p0,0),p0_max);

if p0 == p0_old
    p0 = unifrnd(0,p0_max);
end

options = optimoptions('fmincon','Display','off');
param = fmincon(@(x)fmin(x(1),x(2),x(3),p0),[param(1),param(2),param(3)],[],[],[],[],[0,0,0],[20,20,20],[],options);
    
p1_temp = F_func(param(1),param(2),param(3),p0);
f1_temp = f_func(param(1),param(2),param(3));
f1_pr_temp = f_pr_func(param(1),param(2),param(3));
m_temp = (1/F_func(param(1),param(2),param(3),p0))*integral(@(p)(Q_func(param(1),param(2),p,param(3))),0,F_func(param(1),param(2),param(3),p0)-p0);
        
c_temp = abs(p1_temp-p1)/p1+abs(f1_temp-f1)/f1+abs((f1_pr_temp-f1_pr)/f1_pr)+100*abs(m_temp-mu)/mu;

if c_temp<c_min
	c_min = c_temp;
    Q_low = zeros(size(P_low,1),1);
	Q_low((F_func(param(1),param(2),param(3),p0)/p1)*P_low>p0) = Q_func(param(1),param(2),(F_func(param(1),param(2),param(3),p0)/p1)*P_low((F_func(param(1),param(2),param(3),p0)/p1)*P_low>p0)-p0,param(3));
end
end

%% 7. If p0 is provided
else
    options = optimoptions('fmincon','Display','off');
    param = fmincon(@(x)fmin(x(1),x(2),x(3),p0),[2,1,1],[],[],[],[],[0,0,0],[20,20,20],[],options);
    
    Q_low = zeros(size(P_low,1),1);
    Q_low((F_func(param(1),param(2),param(3),p0)/p1)*P_low>p0) = Q_func(param(1),param(2),(F_func(param(1),param(2),param(3),p0)/p1)*P_low((F_func(param(1),param(2),param(3),p0)/p1)*P_low>p0)-p0,param(3));
end    

% 8. Adjusts for average
% alpha = 10000;
% aux = @(x)(exp(-1./(alpha.*x).^2));
% 
% lambda = 0;
% Q_low_adj = Q_low;
% 
% if trapz(P_low,Q_low_adj)/p1<mu
%     while trapz(P_low,Q_low_adj)/p1 < mu
%         d_Q = [Q_low_adj(2:end); q] - Q_low_adj;
%         df = flip(cumsum(flip(d_Q)));
%         Q_low_adj = Q_low_adj+0.00001*df;
%         mu-trapz(P_low,Q_low_adj)/p1
%     end
% end
% if trapz(P_low,Q_low_adj)/p1<mu
%     while trapz(P_low,Q_low_adj)/p1 < mu
%         lambda = lambda+0.01
%         Q_low_adj = Q_low.*(1+lambda*(P_low/p1).*aux(1-P_low/p1));
%     end
% else
%     while trapz(P_low,Q_low_adj)/p1 > mu
%         lambda = lambda+1;
%         Q_low_adj = Q_low.*(1-lambda*aux(1-P_low/p1));
%     end
% end
% issorted(Q_low_adj)
% Q_low_adj = sort(Q_low_adj);
%Q_low_adj = (Q_low(end)/Q_low_adj(end))*Q_low_adj;
%Q_low_adj(end)
%Q_low_adj = max(Q_low_adj,0);


disp('Lower part estimated.')
