%% Estimation of the quantile function Q(p) using income tax data
%
% Inputs: 
%   * thr: table of incomes corresponding to the thresholds in the tax
%   tabulation
%   * pp: table of percentiles (in [0,1]) corresponding to the thresholds in the tax
%   tabulation
%   * bb: table of (inverted) Pareto coefficients corresponding to the
%   thresholds in the tax tabulation
%   * y_av: average income of the taxpaying population
%   * N: table  of nodes p (in [0,1])
%
% Outputs:
%   * Q: table of approximative values taken by the quantile function Q at
%   the points of table P, that is, Q=Q(N)
%   * p0: approximation of the share of the population with no income
%
% Steps of the program:
%   1. Preliminaries
%   2. Calls function pc.m to estimate the Pareto curve
%   3. Numerically integrates the integral that appears in the formula of Q
%   4. Computes a temporary estimation of Q up to a constant
%   5. Adjusts the estimations of Q and clips it to the tax thresholds
%
%% 1. Preliminaries
% Defines function inputs and outputs.

% thr = tab.thr;
% pp = tab.p;
% bb = tab.b;
% N = u;

%function [Q,B,mid,p0] = quantile_trapz(thr,pp,bb,y_av,N)
function [Q,p0] = quantile_trapz(thr,pp,bb,y_av,N)

T = length(thr);
Xn = length(N);
a = min(N);
b = max(N);

Ym = floor((b-a)*200000000);
mesh = linspace(a,b,Ym)';
mid = (mesh(1:Ym-1,1)+mesh(2:Ym,1))/2;

%% 2. Estimates the Pareto curve
[B,p0] = pc_trapz(thr,pp,bb,y_av,mid);

%% 3.a. Numerical integration - Midpoint rule
% h = mesh(2:Ym)-mesh(1:Ym-1);
% 
% integrand = 1./((1-mid).*B);
% int = cumsum(h.*integrand);

%% 3.b. Numerical integration - Trapezodal rule
integrand = 1./((1-mid).*B);
int = cumtrapz(mid,integrand);%cumsimpsum(mid,B);

%% 4. Computes a temporary estimation of Q up to a constant
temp0 = (1./(B.*(1.-mid))).*exp(-int);

%d_temp = e01be(mid,temp0);
%temp = e01bf(mid,temp0,d_temp,N);
%temp = interp1(mid,temp0,N,'spline','extrap');
temp = nakeinterp1(mid,temp0,N);
clear temp0

%d_q0 =e01be(N,temp);
%q0(1,:) = e01bf(N,temp,d_q0,d_q0);
q0(1,:) = interp1(N,temp,pp(:,1),'spline','extrap')';

Q_mat = temp*(thr'./q0);
% Matrix which columns corresponds to numerical approximations of the
% quantile function Q respectively starting from each threshold of the
% tabulation

%% 5. Adjusts the estimations of Q and clips it to the tax thresholds
Q = zeros(Xn,1);
% Vector which will contain the numerical approximation of the final 
% quantile function Q

%within each bracket, the final value Qfin is a weighted average of the
%quantile functions starting from the two thresholds bounding the bracket
br=0; %bracket
for i=1:Xn
    while (br<T)&&(N(i,1)>pp(br+1,1))
        br = br+1;
    end
    if N(i,1) <= p0
        Q(i,1) = 0;
    elseif br==0
        Q(i,1) = Q_mat(i,1);
        %if below the first threshold, use the quantile function starting 
        %from the first threshold
    elseif br==T
        Q(i,1) = Q_mat(i,T);
        %if after the last threshold, use the quantile function starting 
        %from the last threshold
    else
        w1 = (pp(br+1,1)-N(i,1))/(pp(br+1,1)-pp(br,1));
        w2 = (N(i,1)-pp(br,1))/(pp(br+1,1)-pp(br,1));
        Q(i,1) = w1*Q_mat(i,br)+w2*Q_mat(i,br+1);
        %else, use a weighted average of the two surrounding thresholds
    end
end

clear Q_mat

disp('Quantile function estimated')