%% Simulation of a population of taxpayers using income tax data
%% This code generates total income, capital income and labor income.
% 
% Npop=1000000 => 60 sec
%
% Steps of the program:
%   * 1. Preliminaries
%   * 2. Loads data
%   * 3. Generates a vector of random numbers uniformly distributed on [0,1]
%   * 4. Simulates the population
%   * 5. Joint distribution of capital and labor income
%   * 6. Exports the population
%
%% 1. Preliminaries 
close all; clear all;

currentfolder=pwd;
% Path to load data
loadpath=[currentfolder,'/rawdata'];

% Number of taxpayers to be generated
Npop = 100000;

% Parameter of the Gumbel copula
theta = 1.35;

%% 2. Loads data
disp('Loading data...');

cd(loadpath);
  [rawdata_k] = xlsread('rawdata_gumbel','Capital');
   rawdata_k  = rawdata_k(2:end,:);
  [rawdata_l] = xlsread('rawdata_gumbel','Labor');
   rawdata_l  = rawdata_l(2:end,:);

codespath=[currentfolder,'/codes'];
cd(codespath);
[raw_k.thr,raw_k.p,raw_k.b,yk_av,dirac_k,neg_k] = inputs_check(rawdata_k);
[raw_l.thr,raw_l.p,raw_l.b,yl_av,dirac_l,neg_l] = inputs_check(rawdata_l);

if ~(length(dirac_k)==1)
      p0_k  = dirac_k(2);
	dirac_k = dirac_k(1);
else
    dirac_k = 0;
    p0_k = NaN;
end

if ~(length(dirac_l)==1)
      p0_l  = dirac_l(2);
	dirac_l = dirac_l(1);
else
    dirac_l = 0;
    p0_l = NaN;
end
    
if ~(length(neg_k)==1)
	thr_neg_k = neg_k(1);
	p_neg_k = neg_k(2);
	av_neg_k = neg_k(3);
else
    thr_neg_k = 0;
	p_neg_k = 0;
	av_neg_k = 0;
end

if ~(length(neg_l)==1)
	thr_neg_l = neg_l(1);
	p_neg_l = neg_l(2);
	av_neg_l = neg_l(3);
else
    thr_neg_l = 0;
	p_neg_l = 0;
	av_neg_l = 0;
end
    
clear rawdata_k rawdata_l;
cd(currentfolder);

%% 3. Gumbel copula
disp('Generating the Gumbel copula...');

v = rand(Npop,2);

w = -theta*v(:,2)./lambertw(-theta*exp(-theta)*v(:,2));

u = zeros(Npop,2);
u(:,1) = exp(v(:,1).^(1/theta)*log(w));
u(:,2) = exp((1-v(:,1)).^(1/theta)*log(w));

%% 4. Simulates the population
disp('Simulating the population...');

cd(codespath);
[yk_pop,~] = quantile(shift_p(u(u(:,1)>=p_neg_k,1),p_neg_k),shift_thr(raw_k.thr,dirac_k),shift_p(raw_k.p,p_neg_k),shift_b(raw_k.thr,raw_k.b,dirac_k),shift_av(yk_av,dirac_k,p_neg_k,av_neg_k),shift_p(p_neg_k+p0_k,p_neg_k));
yk_pop = dirac_k+yk_pop;

[yl_pop,~] = quantile(shift_p(u(u(:,2)>=p_neg_l,2),p_neg_l),shift_thr(raw_l.thr,dirac_l),shift_p(raw_l.p,p_neg_l),shift_b(raw_l.thr,raw_l.b,dirac_l),shift_av(yl_av,dirac_l,p_neg_l,av_neg_l),shift_p(p_neg_l+p0_l,p_neg_l));
yl_pop = dirac_l+yl_pop;

y_pop = yk_pop+yl_pop;
sim_pop = [y_pop yk_pop yl_pop];

%% 6. Exports the simulated population
disp('Exporting the simulated population...');

outputpath=[currentfolder,'/output'];
cd(outputpath);
outfile = 'sim_pop_gumbel.csv';

header = {'y','yk','yl'};

fileID = fopen(outfile, 'w');
if fileID == -1; error('Cannot open file: %s', outfile); end
fprintf(fileID, '%s,', header{1:end-1});
fprintf(fileID, '%s\n', header{end});
fprintf(fileID, '%.0f,%.0f,%.0f\n', sim_pop');
fclose(fileID);